import * as tslib_1 from "tslib";
import { deriveColors, deriveColumnDescriptions } from '../provider';
import LocalDataProvider from '../provider/LocalDataProvider';
import LineUp from '../ui/LineUp';
import Taggle from '../ui/taggle/Taggle';
import ColumnBuilder from './column/ColumnBuilder';
import LineUpBuilder from './LineUpBuilder';
import RankingBuilder from './RankingBuilder';
/**
 * builder for a LocalDataProvider along with LineUp configuration options
 */
var DataBuilder = /** @class */ (function (_super) {
    tslib_1.__extends(DataBuilder, _super);
    function DataBuilder(data) {
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.columns = [];
        _this.providerOptions = {
            columnTypes: {}
        };
        _this.rankBuilders = [];
        _this._deriveColors = false;
        return _this;
    }
    /**
     * allow just a single selection
     */
    DataBuilder.prototype.singleSelection = function () {
        this.providerOptions.multiSelection = false;
        return this;
    };
    /**
     * filter all rankings by all filters in LineUp
     */
    DataBuilder.prototype.filterGlobally = function () {
        this.providerOptions.filterGlobally = true;
        return this;
    };
    /**
     * no limits for the number of sorting and grouping criteria
     */
    DataBuilder.prototype.noCriteriaLimits = function () {
        this.providerOptions.maxGroupColumns = Infinity;
        this.providerOptions.maxNestedSortingCriteria = Infinity;
        return this;
    };
    /**
     * limit the number of sort and grouping criterias i.e the nesting level
     * @param {number} sortingCritera
     * @param {number} groupingCriteria
     * @returns {this}
     */
    DataBuilder.prototype.limitCriteria = function (sortingCritera, groupingCriteria) {
        this.providerOptions.maxGroupColumns = sortingCritera;
        this.providerOptions.maxNestedSortingCriteria = groupingCriteria;
        return this;
    };
    /**
     * triggers to derive the column descriptions for the given data
     * @param {string} columns optional enforced order of columns
     */
    DataBuilder.prototype.deriveColumns = function () {
        var columns = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            columns[_i] = arguments[_i];
        }
        var _a;
        (_a = this.columns).push.apply(_a, deriveColumnDescriptions(this.data, { columns: columns }));
        return this;
    };
    /**
     * tirggers to assign colors for the given descriptions
     */
    DataBuilder.prototype.deriveColors = function () {
        this._deriveColors = true;
        return this;
    };
    /**
     * register another column type to this data provider
     * @param {string} type unique type id
     * @param {typeof Column} clazz column class
     */
    DataBuilder.prototype.registerColumnType = function (type, clazz) {
        this.providerOptions.columnTypes[type] = clazz;
        return this;
    };
    /**
     * push another column description to this data provider
     * @param {IColumnDesc | ColumnBuilder} column column description or builder instance
     */
    DataBuilder.prototype.column = function (column) {
        this.columns.push(column instanceof ColumnBuilder ? column.build.bind(column) : column);
        return this;
    };
    /**
     * restores a given ranking dump
     * @param dump dump as created using '.dump()'
     */
    DataBuilder.prototype.restore = function (dump) {
        this.rankBuilders.push(function (data) { return data.restore(dump); });
        return this;
    };
    /**
     * add the default ranking (all columns) to this data provider
     * @param {boolean} addSupportTypes add support types, too, default: true
     */
    DataBuilder.prototype.defaultRanking = function (addSupportTypes) {
        if (addSupportTypes === void 0) { addSupportTypes = true; }
        this.rankBuilders.push(function (data) { return data.deriveDefault(addSupportTypes); });
        return this;
    };
    /**
     * add another ranking to this data provider
     * @param {((data: ADataProvider) => void) | RankingBuilder} builder ranking builder
     */
    DataBuilder.prototype.ranking = function (builder) {
        this.rankBuilders.push(builder instanceof RankingBuilder ? builder.build.bind(builder) : builder);
        return this;
    };
    /**
     * builds the data provider itself
     * @returns {LocalDataProvider}
     */
    DataBuilder.prototype.buildData = function () {
        var _this = this;
        var columns = this.columns.map(function (d) { return typeof d === 'function' ? d(_this.data) : d; });
        if (this._deriveColors) {
            deriveColors(columns);
        }
        var r = new LocalDataProvider(this.data, columns, this.providerOptions);
        if (this.rankBuilders.length === 0) {
            this.defaultRanking();
        }
        this.rankBuilders.forEach(function (builder) { return builder(r); });
        return r;
    };
    /**
     * builds LineUp at the given parent DOM node
     * @param {HTMLElement} node parent DOM node to attach
     * @returns {LineUp}
     */
    DataBuilder.prototype.build = function (node) {
        return new LineUp(node, this.buildData(), this.options);
    };
    /**
     * builds Taggle at the given parent DOM node
     * @param {HTMLElement} node parent DOM node to attach
     * @returns {Taggle}
     */
    DataBuilder.prototype.buildTaggle = function (node) {
        return new Taggle(node, this.buildData(), this.options);
    };
    return DataBuilder;
}(LineUpBuilder));
export default DataBuilder;
/**
 * creates a new builder instance for the given data
 * @param {object[]} arr data to visualize
 * @returns {DataBuilder}
 */
export function builder(arr) {
    return new DataBuilder(arr);
}
//# sourceMappingURL=DataBuilder.js.map