import * as tslib_1 from "tslib";
import { createAggregateDesc, createRankDesc, createSelectionDesc, isSupportType } from '../model';
import Ranking from '../model/Ranking';
import ADataProvider from './ADataProvider';
function isComplexAccessor(column) {
    // something like a.b or a[4]
    return typeof column === 'string' && column.includes('.');
}
function rowComplexGetter(row, _id, desc) {
    var column = desc.column;
    if (row.hasOwnProperty(column)) { // well complex but a direct hit
        return row.v[column];
    }
    var resolve = function (obj, col) {
        if (obj === undefined) {
            return obj; // propagate invalid values
        }
        if (/\d+/.test(col)) { // index
            return obj[+col];
        }
        return obj[col];
    };
    return column.split('.').reduce(resolve, row.v);
}
function rowGetter(row, _id, desc) {
    return row.v[desc.column];
}
function rowGuessGetter(row, _id, desc) {
    var column = desc.column;
    if (isComplexAccessor(column)) {
        return rowComplexGetter(row, _id, desc);
    }
    return rowGetter(row, _id, desc);
}
/**
 * common base implementation of a DataProvider with a fixed list of column descriptions
 */
var ACommonDataProvider = /** @class */ (function (_super) {
    tslib_1.__extends(ACommonDataProvider, _super);
    function ACommonDataProvider(columns, options) {
        if (columns === void 0) { columns = []; }
        if (options === void 0) { options = {}; }
        var _this = _super.call(this, options) || this;
        _this.columns = columns;
        _this.rankingIndex = 0;
        //generate the accessor
        columns.forEach(function (d) {
            d.accessor = d.accessor || (d.column ? (isComplexAccessor(d.column) ? rowComplexGetter : rowGetter) : rowGuessGetter);
            d.label = d.label || d.column;
        });
        return _this;
    }
    ACommonDataProvider.prototype.rankAccessor = function (row, _id, _desc, ranking) {
        var groups = ranking.getGroups();
        for (var _i = 0, groups_1 = groups; _i < groups_1.length; _i++) {
            var group = groups_1[_i];
            var rank = group.index2pos[row.i];
            if (typeof rank === 'number') {
                return rank + 1; // starting with 1
            }
        }
        return -1;
    };
    /**
     * returns the maximal number of nested/hierarchical sorting criteria
     * @return {number}
     */
    ACommonDataProvider.prototype.getMaxNestedSortingCriteria = function () {
        return 1;
    };
    ACommonDataProvider.prototype.getMaxGroupColumns = function () {
        return 1;
    };
    ACommonDataProvider.prototype.cloneRanking = function (existing) {
        var _this = this;
        var id = this.nextRankingId();
        var clone = new Ranking(id, this.getMaxNestedSortingCriteria(), this.getMaxGroupColumns());
        if (existing) { //copy the ranking of the other one
            //TODO better cloning
            existing.children.forEach(function (child) {
                _this.push(clone, child.desc);
            });
        }
        return clone;
    };
    ACommonDataProvider.prototype.sort = function (ranking) {
        //use the server side to sort
        return this.sortImpl(ranking);
    };
    /**
     * adds another column description to this data provider
     * @param column
     */
    ACommonDataProvider.prototype.pushDesc = function (column) {
        var d = column;
        d.accessor = d.accessor || rowGetter;
        d.label = column.label || d.column;
        this.columns.push(column);
        this.fire(ADataProvider.EVENT_ADD_DESC, d);
    };
    ACommonDataProvider.prototype.clearColumns = function () {
        this.clearRankings();
        this.columns.splice(0, this.columns.length);
        this.fire(ADataProvider.EVENT_CLEAR_DESC);
    };
    ACommonDataProvider.prototype.getColumns = function () {
        return this.columns.slice();
    };
    ACommonDataProvider.prototype.findDesc = function (ref) {
        return this.columns.filter(function (c) { return c.column === ref; })[0];
    };
    /**
     * identify by the tuple type@columnname
     * @param desc
     * @returns {string}
     */
    ACommonDataProvider.prototype.toDescRef = function (desc) {
        return typeof desc.column !== 'undefined' ? desc.type + "@" + desc.column : desc;
    };
    /**
     * generates a default ranking by using all column descriptions ones
     */
    ACommonDataProvider.prototype.deriveDefault = function (addSupportType) {
        var _this = this;
        if (addSupportType === void 0) { addSupportType = true; }
        var r = this.pushRanking();
        if (addSupportType) {
            if (this.getMaxGroupColumns() > 0) {
                r.push(this.create(createAggregateDesc()));
            }
            r.push(this.create(createRankDesc()));
            if (this.multiSelections) {
                r.push(this.create(createSelectionDesc()));
            }
        }
        this.getColumns().forEach(function (col) {
            var c = _this.create(col);
            if (!c || isSupportType(c)) {
                return;
            }
            r.push(c);
        });
        return r;
    };
    ACommonDataProvider.prototype.fromDescRef = function (descRef) {
        if (typeof (descRef) === 'string') {
            return this.columns.find(function (d) { return d.type + "@" + d.column === descRef; });
        }
        var existing = this.columns.find(function (d) { return descRef.column === d.column && descRef.label === d.label && descRef.type === d.type; });
        if (existing) {
            return existing;
        }
        return descRef;
    };
    ACommonDataProvider.prototype.restore = function (dump) {
        _super.prototype.restore.call(this, dump);
        this.rankingIndex = 1 + Math.max.apply(Math, this.getRankings().map(function (r) { return +r.id.substring(4); }));
    };
    ACommonDataProvider.prototype.nextRankingId = function () {
        return "rank" + this.rankingIndex++;
    };
    return ACommonDataProvider;
}(ADataProvider));
export default ACommonDataProvider;
//# sourceMappingURL=ACommonDataProvider.js.map