import { ICategoricalStatistics, IStatistics } from '../internal';
import AEventDispatcher from '../internal/AEventDispatcher';
import { Column, ICategoricalColumn, IColumnDesc, IDataRow, IGroup, INumberColumn, IOrderedGroup } from '../model';
import { dirty, dirtyHeader, dirtyValues } from '../model/Column';
import Ranking, { orderChanged, addColumn, removeColumn } from '../model/Ranking';
import { IExportOptions } from './utils';
import { IEventListener } from '../internal/AEventDispatcher';
export { IExportOptions } from './utils';
export interface IStatsBuilder {
    stats(col: INumberColumn, numbrerOfBins?: number): Promise<IStatistics> | IStatistics;
    hist(col: ICategoricalColumn): Promise<ICategoricalStatistics> | ICategoricalStatistics;
}
export interface IDataProviderOptions {
    columnTypes: {
        [columnType: string]: typeof Column;
    };
    /**
     * allow multiple selected rows
     * default: true
     */
    multiSelection: boolean;
}
export interface IDataProvider extends AEventDispatcher {
    readonly columnTypes: {
        [columnType: string]: typeof Column;
    };
    getTotalNumberOfRows(): number;
    takeSnapshot(col: Column): void;
    selectAllOf(ranking: Ranking): void;
    getSelection(): number[];
    setSelection(dataIndices: number[]): void;
    toggleSelection(i: number, additional?: boolean): boolean;
    isSelected(i: number): boolean;
    removeRanking(ranking: Ranking): void;
    ensureOneRanking(): void;
    find(id: string): Column | null;
    clone(col: Column): Column;
    create(desc: IColumnDesc): Column | null;
    toDescRef(desc: IColumnDesc): any;
    fromDescRef(ref: any): IColumnDesc;
    mappingSample(col: Column): Promise<number[]> | number[];
    searchAndJump(search: string | RegExp, col: Column): void;
    getRankings(): Ranking[];
    getFirstRanking(): Ranking | null;
    getLastRanking(): Ranking;
    getColumns(): IColumnDesc[];
    isAggregated(ranking: Ranking, group: IGroup): boolean;
    aggregateAllOf(ranking: Ranking, aggregateAll: boolean): void;
}
/**
 * emitted when the selection changes
 * @asMemberOf ADataProvider
 * @param dataIndices the selected data indices
 * @event
 */
export declare function selectionChanged(dataIndices: number[]): void;
/**
 * @asMemberOf ADataProvider
 * @event
 */
export declare function addRanking(ranking: Ranking, index: number): void;
/**
 * @asMemberOf ADataProvider
 * @param ranking if null all rankings are removed else just the specific one
 * @event
 */
export declare function removeRanking(ranking: Ranking | null, index: number): void;
/**
 * emitted when the selection changes
 * @asMemberOf ADataProvider
 * @param dataIndices the selected data indices
 * @event
 */
export declare function orderchanged(dataIndices: number[]): void;
/**
 * @asMemberOf ADataProvider
 * @event
 */
export declare function addDesc(desc: IColumnDesc): void;
/**
 * @asMemberOf ADataProvider
 * @event
 */
export declare function clearDesc(): void;
/**
 * emitted when the selection changes
 * @asMemberOf ADataProvider
 * @param dataIndices the selected data indices
 * @event
 */
export declare function jumpToNearest(dataIndices: number[]): void;
/**
 * @asMemberOf ADataProvider
 * @event
 */
export declare function aggregate(ranking: Ranking, group: IGroup | IGroup[], value: boolean): void;
/**
 * a basic data provider holding the data and rankings
 */
declare abstract class ADataProvider extends AEventDispatcher implements IDataProvider {
    static readonly EVENT_SELECTION_CHANGED = "selectionChanged";
    static readonly EVENT_ADD_COLUMN = "addColumn";
    static readonly EVENT_REMOVE_COLUMN = "removeColumn";
    static readonly EVENT_ADD_RANKING = "addRanking";
    static readonly EVENT_REMOVE_RANKING = "removeRanking";
    static readonly EVENT_DIRTY = "dirty";
    static readonly EVENT_DIRTY_HEADER = "dirtyHeader";
    static readonly EVENT_DIRTY_VALUES = "dirtyValues";
    static readonly EVENT_ORDER_CHANGED = "orderChanged";
    static readonly EVENT_ADD_DESC = "addDesc";
    static readonly EVENT_CLEAR_DESC = "clearDesc";
    static readonly EVENT_JUMP_TO_NEAREST = "jumpToNearest";
    static readonly EVENT_GROUP_AGGREGATION_CHANGED = "aggregate";
    /**
     * all rankings
     * @type {Array}
     * @private
     */
    private rankings;
    /**
     * the current selected indices
     * @type {OrderedSet}
     */
    private readonly selection;
    private aggregations;
    private uid;
    /**
     * lookup map of a column type to its column implementation
     */
    readonly columnTypes: {
        [columnType: string]: typeof Column;
    };
    protected readonly multiSelections: boolean;
    constructor(options?: Partial<IDataProviderOptions>);
    /**
     * events:
     *  * column changes: addColumn, removeColumn
     *  * ranking changes: addRanking, removeRanking
     *  * dirty: dirty, dirtyHeder, dirtyValues
     *  * selectionChanged
     * @returns {string[]}
     */
    protected createEventList(): string[];
    on(type: typeof ADataProvider.EVENT_ADD_COLUMN, listener: typeof addColumn | null): this;
    on(type: typeof ADataProvider.EVENT_REMOVE_COLUMN, listener: typeof removeColumn | null): this;
    on(type: typeof ADataProvider.EVENT_ADD_RANKING, listener: typeof addRanking | null): this;
    on(type: typeof ADataProvider.EVENT_REMOVE_RANKING, listener: typeof removeRanking | null): this;
    on(type: typeof ADataProvider.EVENT_DIRTY, listener: typeof dirty | null): this;
    on(type: typeof ADataProvider.EVENT_DIRTY_HEADER, listener: typeof dirtyHeader | null): this;
    on(type: typeof ADataProvider.EVENT_DIRTY_VALUES, listener: typeof dirtyValues | null): this;
    on(type: typeof ADataProvider.EVENT_ORDER_CHANGED, listener: typeof orderChanged | null): this;
    on(type: typeof ADataProvider.EVENT_ADD_DESC, listener: typeof addDesc | null): this;
    on(type: typeof ADataProvider.EVENT_CLEAR_DESC, listener: typeof clearDesc | null): this;
    on(type: typeof ADataProvider.EVENT_JUMP_TO_NEAREST, listener: typeof jumpToNearest | null): this;
    on(type: typeof ADataProvider.EVENT_GROUP_AGGREGATION_CHANGED, listener: typeof aggregate | null): this;
    on(type: typeof ADataProvider.EVENT_SELECTION_CHANGED, listener: typeof selectionChanged | null): this;
    on(type: string | string[], listener: IEventListener | null): this;
    abstract getTotalNumberOfRows(): number;
    /**
     * returns a list of all known column descriptions
     * @returns {Array}
     */
    abstract getColumns(): IColumnDesc[];
    /**
     * adds a new ranking
     * @param existing an optional existing ranking to clone
     * @return the new ranking
     */
    pushRanking(existing?: Ranking): Ranking;
    takeSnapshot(col: Column): Ranking;
    insertRanking(r: Ranking, index?: number): void;
    protected triggerReorder(ranking: Ranking): void;
    /**
     * removes a ranking from this data provider
     * @param ranking
     * @returns {boolean}
     */
    removeRanking(ranking: Ranking): boolean;
    /**
     * removes all rankings
     */
    clearRankings(): void;
    /**
     * returns a list of all current rankings
     * @returns {Ranking[]}
     */
    getRankings(): Ranking[];
    /**
     * returns the last ranking for quicker access
     * @returns {Ranking}
     */
    getFirstRanking(): Ranking;
    /**
     * returns the last ranking for quicker access
     * @returns {Ranking}
     */
    getLastRanking(): Ranking;
    ensureOneRanking(): void;
    /**
     * hook method for cleaning up a ranking
     * @param _ranking
     */
    cleanUpRanking(_ranking: Ranking): void;
    /**
     * abstract method for cloning a ranking
     * @param existing
     * @returns {null}
     */
    abstract cloneRanking(existing?: Ranking): Ranking;
    /**
     * adds a column to a ranking described by its column description
     * @param ranking the ranking to add the column to
     * @param desc the description of the column
     * @return {Column} the newly created column or null
     */
    push(ranking: Ranking, desc: IColumnDesc): Column | null;
    /**
     * adds a column to a ranking described by its column description
     * @param ranking the ranking to add the column to
     * @param index the position to insert the column
     * @param desc the description of the column
     * @return {Column} the newly created column or null
     */
    insert(ranking: Ranking, index: number, desc: IColumnDesc): Column | null;
    /**
     * creates a new unique id for a column
     * @returns {string}
     */
    private nextId;
    protected abstract rankAccessor(row: IDataRow, id: string, desc: IColumnDesc, ranking: Ranking): number;
    private fixDesc;
    /**
     * creates an internal column model out of the given column description
     * @param desc
     * @returns {Column} the new column or null if it can't be created
     */
    create(desc: IColumnDesc): Column | null;
    /**
     * clones a column by dumping and restoring
     * @param col
     * @returns {Column}
     */
    clone(col: Column): Column;
    /**
     * restores a column from a dump
     * @param dump
     * @returns {Column}
     */
    restoreColumn(dump: any): Column;
    /**
     * finds a column in all rankings returning the first match
     * @param idOrFilter by id or by a filter function
     * @returns {Column}
     */
    find(idOrFilter: string | ((col: Column) => boolean)): Column | null;
    /**
     * dumps this whole provider including selection and the rankings
     * @returns {{uid: number, selection: number[], rankings: *[]}}
     */
    dump(): any;
    /**
     * dumps a specific column
     */
    dumpColumn(col: Column): any;
    /**
     * for better dumping describe reference, by default just return the description
     */
    toDescRef(desc: any): any;
    /**
     * inverse operation of toDescRef
     */
    fromDescRef(descRef: any): any;
    private createHelper;
    restoreRanking(dump: any): Ranking;
    restore(dump: any): void;
    abstract findDesc(ref: string): IColumnDesc | null;
    abstract deriveDefault(addSupporType?: boolean): Ranking;
    /**
     * derives a ranking from an old layout bundle format
     * @param bundle
     */
    private deriveRanking;
    isAggregated(ranking: Ranking, group: IGroup): boolean;
    private unaggregateParents;
    setAggregated(ranking: Ranking, group: IGroup, value: boolean): void;
    aggregateAllOf(ranking: Ranking, aggregateAll: boolean): void;
    /**
     * sorts the given ranking and eventually return a ordering of the data items
     * @param ranking
     * @return {Promise<any>}
     */
    abstract sort(ranking: Ranking): Promise<IOrderedGroup[]> | IOrderedGroup[];
    /**
     * returns a view in the order of the given indices
     * @param indices
     * @return {Promise<any>}
     */
    abstract view(indices: number[]): Promise<any[]> | any[];
    abstract fetch(orders: number[][]): (Promise<IDataRow> | IDataRow)[][];
    /**
     * returns a data sample used for the mapping editor
     * @param col
     * @return {Promise<any>}
     */
    abstract mappingSample(col: Column): Promise<number[]> | number[];
    /**
     * helper for computing statistics
     * @param indices
     * @returns {{stats: (function(INumberColumn): *), hist: (function(ICategoricalColumn): *)}}
     */
    abstract stats(indices?: number[]): IStatsBuilder;
    /**
     * is the given row selected
     * @param index
     * @return {boolean}
     */
    isSelected(index: number): boolean;
    /**
     * also select the given row
     * @param index
     */
    select(index: number): void;
    /**
     * hook for selecting elements matching the given arguments
     * @param search
     * @param col
     */
    abstract searchAndJump(search: string | RegExp, col: Column): void;
    jumpToNearest(indices: number[]): void;
    /**
     * also select all the given rows
     * @param indices
     */
    selectAll(indices: number[]): void;
    selectAllOf(ranking: Ranking): void;
    /**
     * set the selection to the given rows
     * @param indices
     */
    setSelection(indices: number[]): void;
    /**
     * toggles the selection of the given data index
     * @param index
     * @param additional just this element or all
     * @returns {boolean} whether the index is currently selected
     */
    toggleSelection(index: number, additional?: boolean): boolean;
    /**
     * deselect the given row
     * @param index
     */
    deselect(index: number): void;
    /**
     * also select all the given rows
     * @param indices
     */
    deselectAll(indices: number[]): void;
    /**
     * returns a promise containing the selected rows
     * @return {Promise<any[]>}
     */
    selectedRows(): Promise<any[]> | any[];
    /**
     * returns the currently selected indices
     * @returns {Array}
     */
    getSelection(): number[];
    /**
     * clears the selection
     */
    clearSelection(): void;
    /**
     * utility to export a ranking to a table with the given separator
     * @param ranking
     * @param options
     * @returns {Promise<string>}
     */
    exportTable(ranking: Ranking, options?: Partial<IExportOptions>): Promise<string>;
}
export default ADataProvider;
