import { extent } from 'd3-array';
import { isNumberColumn, isSupportType, isMapAbleColumn } from '../model';
import Column from '../model/Column';
import { colorPool } from '../model/internal';
/**
 * @internal
 */
export function cleanCategories(categories) {
    // remove missing values
    categories.delete(null);
    categories.delete(undefined);
    categories.delete('');
    categories.delete('NA');
    categories.delete('NaN');
    categories.delete('na');
    return Array.from(categories).map(String).sort();
}
function deriveType(label, value, column, data, options) {
    var _a, _b;
    var base = {
        type: 'string',
        label: label,
        column: column,
    };
    if (typeof value === 'number') {
        base.type = 'number';
        base.domain = extent(data, function (d) { return d[column]; });
        return base;
    }
    if (value && value instanceof Date) {
        base.type = 'date';
        return base;
    }
    if (typeof value === 'boolean') {
        base.type = 'boolean';
        return base;
    }
    if (typeof value === 'string') {
        //maybe a categorical
        var categories = new Set(data.map(function (d) { return d[column]; }));
        if (categories.size < data.length * options.categoricalThreshold) { // 70% unique guess categorical
            base.type = 'categorical';
            base.categories = cleanCategories(categories);
        }
        return base;
    }
    if (Array.isArray(value)) {
        base.type = 'strings';
        base.dataLength = value.length;
        var vs = value[0];
        if (typeof vs === 'number') {
            base.type = 'numbers';
            base.domain = extent((_a = []).concat.apply(_a, data.map(function (d) { return d[column]; })));
            return base;
        }
        if (vs && value instanceof Date) {
            base.type = 'dates';
            return base;
        }
        if (typeof value === 'boolean') {
            base.type = 'booleans';
            return base;
        }
        if (typeof value === 'string') {
            //maybe a categorical
            var categories = new Set((_b = []).concat.apply(_b, data.map(function (d) { return d[column]; })));
            if (categories.size < data.length * options.categoricalThreshold) { // 70% unique guess categorical
                base.type = 'categoricals';
                base.categories = cleanCategories(categories);
            }
            return base;
        }
    }
    console.log('cannot infer type of column:', column);
    //unknown type
    return base;
}
export function deriveColumnDescriptions(data, options) {
    if (options === void 0) { options = {}; }
    var config = Object.assign({
        categoricalThreshold: 0.7,
        columns: []
    }, options);
    var r = [];
    if (data.length === 0) {
        // no data to derive something from
        return r;
    }
    var first = data[0];
    if (Array.isArray(first)) {
        //array of arrays
        return first.map(function (v, i) { return deriveType("Col" + i, v, i, data, config); });
    }
    //objects
    var columns = config.columns.length > 0 ? config.columns : Object.keys(first);
    return columns.map(function (key) { return deriveType(key, first[key], key, data, config); });
}
/**
 * assigns colors to columns if they are numbers and not yet defined
 * @param columns
 * @returns {IColumnDesc[]}
 */
export function deriveColors(columns) {
    var colors = colorPool();
    columns.forEach(function (col) {
        if (isMapAbleColumn(col)) {
            col.colorMapping = col.colorMapping || col.color || colors() || Column.DEFAULT_COLOR;
        }
    });
    return columns;
}
/**
 * utility to export a ranking to a table with the given separator
 * @param ranking
 * @param data
 * @param options
 * @returns {Promise<string>}
 */
export function exportRanking(ranking, data, options) {
    if (options === void 0) { options = {}; }
    var opts = Object.assign({
        separator: '\t',
        newline: '\n',
        header: true,
        quote: false,
        quoteChar: '"',
        filter: function (c) { return !isSupportType(c); },
        verboseColumnHeaders: false
    }, options);
    //optionally quote not numbers
    var escape = new RegExp("[" + opts.quoteChar + "]", 'g');
    function quote(v, c) {
        var l = String(v);
        if ((opts.quote || l.indexOf('\n') >= 0) && (!c || !isNumberColumn(c))) {
            return "" + opts.quoteChar + l.replace(escape, opts.quoteChar + opts.quoteChar) + opts.quoteChar;
        }
        return l;
    }
    var columns = ranking.flatColumns.filter(function (c) { return opts.filter(c); });
    var order = ranking.getOrder();
    var r = [];
    if (opts.header) {
        r.push(columns.map(function (d) { return quote("" + d.label + (opts.verboseColumnHeaders && d.description ? "\n" + d.description : '')); }).join(opts.separator));
    }
    data.forEach(function (row, i) {
        r.push(columns.map(function (c) { return quote(c.getExportValue({ v: row, i: order[i] }, 'text'), c); }).join(opts.separator));
    });
    return r.join(opts.newline);
}
//# sourceMappingURL=utils.js.map