import { DENSE_HISTOGRAM } from '../config';
import { computeHist, round } from '../internal/math';
import { isCategoricalColumn } from '../model';
import CategoricalColumn from '../model/CategoricalColumn';
import { isCategoryIncluded } from '../model/ICategoricalColumn';
import OrdinalColumn from '../model/OrdinalColumn';
import { CANVAS_HEIGHT, cssClass, FILTERED_OPACITY } from '../styles';
import { filterMissingNumberMarkup, updateFilterMissingNumberMarkup } from '../ui/missing';
import { renderMissingCanvas, renderMissingDOM } from './missing';
import { setText, wideEnough, forEach } from './utils';
import { color } from 'd3-color';
/** @internal */
var CategoricalCellRenderer = /** @class */ (function () {
    function CategoricalCellRenderer() {
        this.title = 'Color';
        this.groupTitle = 'Histogram';
    }
    CategoricalCellRenderer.prototype.canRender = function (col) {
        return isCategoricalColumn(col);
    };
    CategoricalCellRenderer.prototype.create = function (col, context) {
        var width = context.colWidth(col);
        return {
            template: "<div>\n        <div class=\"" + cssClass('cat-color') + "\"></div><div class=\"" + cssClass('cat-label') + "\"></div>\n      </div>",
            update: function (n, d) {
                renderMissingDOM(n, col, d);
                var v = col.getCategory(d);
                n.firstElementChild.style.backgroundColor = v ? v.color : null;
                setText(n.lastElementChild, col.getLabel(d));
            },
            render: function (ctx, d) {
                if (renderMissingCanvas(ctx, col, d, width)) {
                    return;
                }
                var v = col.getCategory(d);
                ctx.fillStyle = v ? v.color : '';
                ctx.fillRect(0, 0, width, CANVAS_HEIGHT);
            }
        };
    };
    CategoricalCellRenderer.prototype.createGroup = function (col, _context, globalHist) {
        var _a = hist(col, false, null), template = _a.template, update = _a.update;
        return {
            template: template + "</div>",
            update: function (n, _group, rows) {
                var _a = computeHist(rows, function (r) { return col.getCategory(r); }, col.categories), maxBin = _a.maxBin, hist = _a.hist;
                var max = Math.max(maxBin, globalHist ? globalHist.maxBin : 0);
                update(n, max, hist);
            }
        };
    };
    CategoricalCellRenderer.prototype.createSummary = function (col, ctx, interactive, unfilteredHist) {
        return (col instanceof CategoricalColumn || col instanceof OrdinalColumn) ? interactiveSummary(col, interactive, unfilteredHist, ctx.idPrefix) : staticSummary(col, interactive);
    };
    return CategoricalCellRenderer;
}());
export default CategoricalCellRenderer;
function staticSummary(col, interactive) {
    var _a = hist(col, interactive, null), template = _a.template, update = _a.update;
    return {
        template: template + "</div>",
        update: function (n, hist) {
            n.classList.toggle(cssClass('missing'), !hist);
            if (!hist) {
                return;
            }
            update(n, hist.maxBin, hist.hist);
        }
    };
}
function interactiveSummary(col, interactive, unfilteredHist, idPrefix) {
    var _a = hist(col, interactive || wideEnough(col), interactive ? unfilteredHist : null), template = _a.template, update = _a.update;
    var filterUpdate;
    return {
        template: "" + template + (interactive ? filterMissingNumberMarkup(false, 0, idPrefix) : '') + "</div>",
        update: function (n, hist) {
            if (!filterUpdate) {
                filterUpdate = interactiveHist(col, n);
            }
            filterUpdate((interactive && unfilteredHist) ? unfilteredHist.missing : (hist ? hist.missing : 0), col);
            n.classList.toggle(cssClass('missing'), !hist);
            if (!hist) {
                return;
            }
            update(n, hist.maxBin, hist.hist);
        }
    };
}
function hist(col, showLabels, unfilteredHist) {
    var bins = col.categories.map(function (c) { return "<div class=\"" + cssClass('histogram-bin') + "\" title=\"" + c.label + ": 0\" data-cat=\"" + c.name + "\" " + (showLabels ? "data-title=\"" + c.label + "\"" : '') + "><div style=\"height: 0; background-color: " + c.color + "\"></div></div>"; }).join('');
    var template = "<div class=\"" + cssClass('histogram') + " " + (col.dataLength > DENSE_HISTOGRAM ? cssClass('dense') : '') + "\">" + bins; // no closing div to be able to append things
    var selected = col.categories.map(function (d) {
        var c = color(d.color);
        c.opacity = FILTERED_OPACITY;
        return c.toString();
    });
    return {
        template: template,
        update: function (n, lMaxBin, hist) {
            var gHist = unfilteredHist ? unfilteredHist.hist : null;
            var maxBin = unfilteredHist ? unfilteredHist.maxBin : lMaxBin;
            forEach(n, '[data-cat]', function (d, i) {
                var cat = col.categories[i];
                var y = hist[i].y;
                var inner = d.firstElementChild;
                if (gHist) {
                    var gY = gHist[i].y;
                    d.title = cat.label + ": " + y + " of " + gY;
                    inner.style.height = round(gY * 100 / maxBin, 2) + "%";
                    var relY = 100 - round(y * 100 / gY, 2);
                    inner.style.background = relY === 0 ? cat.color : (relY === 100 ? selected[i] : "linear-gradient(" + selected[i] + " " + relY + "%, " + cat.color + " " + relY + "%, " + cat.color + " 100%)");
                }
                else {
                    d.title = col.categories[i].label + ": " + y;
                    var inner_1 = d.firstElementChild;
                    inner_1.style.height = Math.round(y * 100 / maxBin) + "%";
                }
            });
        }
    };
}
/** @internal */
export function interactiveHist(col, node) {
    var bins = Array.from(node.querySelectorAll('[data-cat]'));
    bins.forEach(function (bin, i) {
        var cat = col.categories[i];
        bin.onclick = function (evt) {
            evt.preventDefault();
            evt.stopPropagation();
            // toggle filter
            var old = col.getFilter();
            if (old == null || !Array.isArray(old.filter)) {
                // deselect
                var included = col.categories.slice();
                bin.dataset.filtered = '';
                included.splice(i, 1);
                col.setFilter({
                    filterMissing: old ? old.filterMissing : false,
                    filter: included.map(function (d) { return d.name; })
                });
                return;
            }
            var filter = old.filter.slice();
            var contained = filter.indexOf(cat.name);
            if (contained >= 0) {
                // remove
                bin.dataset.filtered = '';
                filter.splice(contained, 1);
            }
            else {
                // readd
                delete bin.dataset.filtered;
                filter.push(cat.name);
            }
            if (!old.filterMissing && filter.length === col.categories.length) {
                // dummy filter
                col.setFilter(null);
                return;
            }
            col.setFilter({
                filterMissing: old.filterMissing,
                filter: filter
            });
        };
    });
    var filterMissing = node.querySelector('input');
    if (filterMissing) {
        filterMissing.onchange = function () {
            // toggle filter
            var v = filterMissing.checked;
            var old = col.getFilter();
            if (old == null) {
                col.setFilter(v ? { filterMissing: v, filter: col.categories.map(function (d) { return d.name; }) } : null);
            }
            else if (!v && Array.isArray(old.filter) && old.filter.length === col.categories.length) {
                // dummy
                col.setFilter(null);
            }
            else {
                col.setFilter({ filterMissing: v, filter: old.filter });
            }
        };
    }
    return function (missing, actCol) {
        col = actCol;
        var cats = col.categories;
        var f = col.getFilter();
        bins.forEach(function (bin, i) {
            if (!isCategoryIncluded(f, cats[i])) {
                bin.dataset.filtered = '';
            }
            else {
                delete bin.dataset.filtered;
            }
        });
        if (filterMissing) {
            filterMissing.checked = f != null && f.filterMissing;
            updateFilterMissingNumberMarkup(filterMissing.parentElement, missing);
        }
    };
}
//# sourceMappingURL=CategoricalCellRenderer.js.map